#!/usr/bin/python
# -*- coding: utf-8 -*-

""" Run tradeshow_matches.py first, and it will convert the tradeshow input 
file to the matches file. Then review the matches file manually and use 
this script to import the two files.
"""

from client import client
import codecs
import csv
from datetime import datetime
import sys
from traceback import format_exception_only

tradeshow_filename = "Neuroscience_2012.csv"
matches_filename = "Neuroscience_2012_matches.csv"
matches = {}
fmatches = codecs.open(matches_filename,"r", encoding='latin1')
match_reader = csv.DictReader(fmatches)
for row in match_reader:
    partner_id = row['partner1_id'] or row['partner2_id'] or row['partner3_id'] or row['partner4_id']
    if partner_id:
        matches[row['New Partner']] = int(partner_id)

fin = codecs.open(tradeshow_filename,"r", encoding='latin1')
reader = csv.reader(fin)

# Load all partner names

c = client()

user_map = {'BRYAN': 32, 'JESSE': 40}
countries = c.search_fetch(
    'res.country',
    [],
    ['name', 'id'])
country_map = {}
for country in countries:
    country_map[country['name']] = country['id']

states = c.search_fetch(
    'res.country.state',
    [],
    ['code', 'country_id', 'id'])
state_map = {}
for state in states:
    state_map['%d|%s' % (state['country_id'][0], state['code'].upper())] = state['id']

def create_partner(visit_datetime, name):
    """ Create a new database record for the partner.
    
    Returns the new partner id.
    """
    return c.create(
        'res.partner', 
        {
         'name': name,
         'date': visit_datetime.strftime('%Y-%m-%d')
        })

IGNORED_STATES = (
    '101|TAM', #India
    '101|GUJ',
    '101|KAR',
    '117|SEO', #Korea
    '150|PUE', #Mexico
    '74|0', #France
)
ALTERNATE_COUNTRIES = {
    'CANADA': 'Canada', 
    'MALAYSIA': 'Malaysia'
}

NOTE_COLUMNS = set((
    "Neuroscience Sub-Discipline",
    "Primary Interest",
    "Secondary Interest",
    "Which Best Describes Your Type of Employer?",
    "Product Category Interest",
    "Purchase Role",
    "Types of products do you use in your research or practice?"))

def create_contact(row, partner_id, contact_name, raw_row, column_names):
    notes = ''
    for i in range(len(raw_row)):
        column_name = column_names[i]
        column_value = raw_row[i].strip()
        if column_name in NOTE_COLUMNS and column_value:
            if not (column_name.endswith('?') or column_name.endswith(':')):
                column_name += ':'
            notes += column_name + ' ' + column_value + '\n'
    
    country_name = row['Country'].strip()
    country_name = ALTERNATE_COUNTRIES.get(country_name, country_name)
    if country_name:
        country_id = country_map[country_name]
    else:
        country_id = country_map['United States']
    state_code = row['State / Province'].strip()
    if country_id and state_code:
        state_key = '%d|%s' % (country_id, state_code.upper())
        if state_key not in IGNORED_STATES:
            state_id = state_map[state_key]
        else:
            state_id = False
    else:
        state_id = False
    return c.create(
        'res.partner.address',
        {
         'name': contact_name,
         'street': row['Street'] or False,
         'street2': row['Street2'] or False,
         'phone': row['Phone'] or False,
         'partner_id': partner_id,
         'city': row['City'] or False,
         'zip': row['Zipcode'] or False,
         'notes': notes or False,
         'country_id': country_id,
         'state_id': state_id,
         'email': row['E-mail'] or False,
         'where_found': 'Neuroscience 2012'
        })

def find_contact(row, contact_name, partner_id):
    contact_ids = c.search(
        'res.partner.address', 
        [
         ('partner_id', '=', partner_id),
         ('name', '=', contact_name),
         ('email', '=', row['E-mail'])
        ])
    if contact_ids:
        return contact_ids[0]
    return None

GENERAL_INFO = """\
Thank you for visiting Zaber's booth at Neuroscience.

To refresh your memory, Zaber manufactures computer-controlled
positioning equipment (linear and rotary stages, actuators, gantries,
xyz systems, etc...) with built-in controllers. Our devices
daisy-chain together, and they can be controlled from a single serial
port. We also provide free, versatile software and source code in
several programming languages.

When we met, you asked me to send you some general information.

Here are a few links to general information on our website:

    * Overview of our products: http://zaber.com/products - pricing can be found at each product's main page.
    * 30-day satisfaction guarantee in case you would like to try out any of our products: http://zaber.com/ordering/?tab=Policies#tabs
    * Order or download a catalogue: http://zaber.com/products/product_accessories.php?detail=CATALOGUE

Best regards,

Bryan

PS: If you sign up to receive Zaber's newsletter, you will be able to
access discounts on new and existing products. These discounts are
offered only to newsletter subscribers. Don't worry, we only send a
few newsletters per year, so we won't fill your inbox with spam. You
can sign up here if you are interested:
http://zaber.com/news/?tab=Newsletter&act=subscribe
"""

FLAG_COLUMNS = set((
    "MAIL CATALOG",
    "SEND GENERAL INFO",
    "OEM",
    "HIGH VALVE",
    "T-LSM",
    "T-NA",
    "ASR",
    "LAC",
    "A-BLQ-E",
    "A-LSQ-E",
    "A-LSQ",
    "A-LST",
    "A-LST-C",
    "T-RS",
    "T-DMG",
    "GANTRY",
    "XYZ",
    "VACUUM"))

def create_case(row, partner_id, contact_id, visit_datetime):
    converted_datetime = visit_datetime.strftime('%Y-%m-%d %H:%M:%S')
    responsibles = filter(None, [row[user] == 'Y' and user 
                                 for user in user_map.keys()])
    responsible_id = responsibles and user_map[responsibles[0]] or False
    descriptions = [row[col_name] == 'Y' and col_name 
                    for col_name in FLAG_COLUMNS]
    descriptions.append(row['Notes'].replace('<br>', '\n').strip())
    description = '\n'.join(filter(None, descriptions))
    description = description or 'No boxes checked'
    email = row['E-mail']
    add_template = (
        row['MAIL CATALOG'] == 'Y' or 
        row['SEND GENERAL INFO'] == 'Y')
    
    case_id = c.create(
        'crm.case',
        {
         'name': 'Neuroscience 2012',
         'state': 'open',
         'date_action_last': converted_datetime,
         'date': converted_datetime,
         'partner_id': partner_id,
         'partner_address_id': contact_id,
         'user_id': responsible_id,
         'email_from': email,
         'section_id': 2,
        })
    log_id = c.create(
        'crm.case.log',
        {
         'case_id': case_id,
         'user_id': responsible_id,
         'name': 'Historize',
         'date': converted_datetime,
         'section_id': 2
        })
    c.create(
        'crm.case.history',
        {
         'log_id': log_id,
         'description': description,
         'email': email
        })
    if add_template:
        log_id = c.create(
            'crm.case.log',
            {
             'case_id': case_id,
             'user_id': responsible_id,
             'name': 'Historize',
             'date': converted_datetime,
             'section_id': 2
            })
        c.create(
            'crm.case.history',
            {
             'log_id': log_id,
             'description': GENERAL_INFO,
             'email': email
            })

ignored_companies = ('Zaber Technologies Inc.',)
fieldnames = reader.next() # Read first row.

rownum = 2
try:
    for raw_row in reader:
        row = {}
        for column, value in map(None, fieldnames, raw_row):
            if column not in row or not row[column]:
                row[column] = value
            elif value:
                raise 'Two values for column %s on line %d.' % (
                    column, 
                    reader.line_num)
                    
        new_name = row['Company'].strip()
        if new_name in ignored_companies:
            continue
        contact_name = row['First Name'] + ' ' + row['Last Name']
        if not new_name:
            new_name = contact_name
        visit_datetime = datetime.strptime(
            row['Scan_TS'], 
            '%m-%d-%Y %H:%M:%S')
    
        partner_id = matches.get(new_name)
        if not partner_id:
            partner_id = create_partner(visit_datetime, new_name)
            matches[new_name] = partner_id
            contact_id = None
        else:
            contact_id = find_contact(row, contact_name, partner_id)
        if not contact_id:
            contact_id = create_contact(
                row, 
                partner_id, 
                contact_name, 
                raw_row, 
                fieldnames)
        create_case(row, partner_id, contact_id, visit_datetime)
except Exception:
    extype, ex, traceback = sys.exc_info()
    formatted = format_exception_only(extype, ex)[-1]
    message = "Importing row %d, %s" % (rownum, formatted)
    raise RuntimeError, message, traceback
print 'Success.'
